@@ -1,6 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
+import logging
 import time
 import uuid
 from email.header import Header
@@ -31,7 +32,7 @@ from trytond.report import Report
 from trytond.sendmail import sendmail_transactional, SMTPDataManager
 from trytond.tools import grouped_slice, reduce_ids
 from trytond.transaction import Transaction
-from trytond.url import HOSTNAME
+from trytond.url import http_host
 from trytond.wsgi import Base64Converter
 
 from .exceptions import DomainError, ConditionError, TemplateError
@@ -44,10 +45,9 @@ if not config.get(
         'fullpage')
 
 USE_SSL = bool(config.get('ssl', 'certificate'))
-URL_BASE = config.get('marketing', 'automation_base',
-    default=urlunsplit(
-        ('http' + ('s' if USE_SSL else ''), HOSTNAME, '', '', '')))
+URL_BASE = config.get('marketing', 'automation_base', default=http_host())
 URL_OPEN = urljoin(URL_BASE, '/m/empty.gif')
+logger = logging.getLogger(__name__)
 
 
 def _formataddr(name, email):
@@ -134,7 +134,7 @@ class Scenario(Workflow, ModelSQL, ModelView):
             else:
                 others.append(scenario)
 
-        count = {name: dict.fromkeys(map(int, scenarios), 0) for name in names}
+        count = {name: dict.fromkeys(map(int, scenarios)) for name in names}
         for sub in grouped_slice(others):
             cursor.execute(*record.select(
                     record.scenario,
@@ -150,8 +150,11 @@ class Scenario(Workflow, ModelSQL, ModelView):
         for scenario in drafts:
             Model = pool.get(scenario.model)
             domain = PYSONDecoder({}).decode(scenario.domain)
-            count['record_count'][scenario.id] = Model.search(
-                domain, count=True)
+            try:
+                count['record_count'][scenario.id] = Model.search(
+                    domain, count=True)
+            except Exception:
+                pass
         return count
 
     @classmethod
@@ -161,10 +164,13 @@ class Scenario(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def check_domain(cls, scenarios):
+        pool = Pool()
         for scenario in scenarios:
+            Model = pool.get(scenario.model)
             try:
                 value = PYSONDecoder({}).decode(scenario.domain)
                 fields.domain_validate(value)
+                Model.search(value, limit=0)
             except Exception as exception:
                 raise DomainError(
                     gettext('marketing_automation.msg_scenario_invalid_domain',
@@ -206,9 +212,15 @@ class Scenario(Workflow, ModelSQL, ModelView):
             record = Record.__table__()
             cursor = Transaction().connection.cursor()
             domain = PYSONDecoder({}).decode(scenario.domain)
+            try:
+                query = Model.search(domain, query=True, order=[])
+            except Exception:
+                logger.error(
+                    "Error when triggering scenario %d", scenario.id,
+                    exc_info=True)
+                continue
             cursor.execute(*(
-                    Model.search(domain, query=True, order=[])
-                    - record.select(
+                    query - record.select(
                         Substring(
                             record.record,
                             Position(',', record.record) + Literal(1)
@@ -274,7 +286,7 @@ class Activity(ModelSQL, ModelView):
     condition = fields.Char("Condition",
         help="The PYSON statement that the record must match "
         "in order to execute the activity.\n"
-        'The record is represented by "self"')
+        'The record is represented by "self".')
 
     delay = fields.TimeDelta(
         "Delay",
@@ -464,10 +476,6 @@ class Activity(ModelSQL, ModelView):
         RecordActivity = pool.get('marketing.automation.record.activity')
         record = activity.record
 
-        # As it is a reference, the record may have been deleted
-        if not record:
-            return
-
         # XXX: use domain
         if self.condition and not record.eval(self.condition):
             return
@@ -662,10 +670,7 @@ class Record(ModelSQL, ModelView):
         cls.write(records, {'blocked': True})
 
     def get_rec_name(self, name):
-        if self.record:
-            return self.record.rec_name
-        else:
-            return '(%s)' % self.id
+        return self.record.rec_name
 
     @classmethod
     def create(cls, vlist):
@@ -838,10 +843,9 @@ class RecordActivity(Workflow, ModelSQL, ModelView):
         cls._cancel_opposite(record_activities)
 
         now = datetime.datetime.now()
-        smtpd_datamanager = Transaction().join(SMTPDataManager())
         for record_activity in record_activities:
             record_activity.activity.execute(
-                record_activity, smtpd_datamanager=smtpd_datamanager, **kwargs)
+                record_activity, smtpd_datamanager=SMTPDataManager(), **kwargs)
             record_activity.at = now
             record_activity.state = 'done'
         cls.save(record_activities)
